#!perl -w
use Test::More tests => 20;
use strict;

use DE_EPAGES::Core::API::Number qw(
    ParseIntegerRange
    ElementOfIntegerRange
    ParseIntegerList
    ElementOfIntegerList
);
use DE_EPAGES::Core::API::Script qw(RunScript);
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );

## no critic (ProhibitMagicNumbers)
sub _TestIntegerRange {
    my ($Min, $Max) = ParseIntegerRange('23-42');
    is($Min, 23, 'ParseIntegerRange min 23');
    is($Max, 42, 'ParseIntegerRange max 42');

    ($Min, $Max) = ParseIntegerRange("     -23     -\t \t23    ");
    is($Min, -23, 'ParseIntegerRange min -23');
    is($Max, 23, 'ParseIntegerRange max 23');

    ($Min, $Max) = ParseIntegerRange('-42--23');
    is($Min, -42, 'ParseIntegerRange min -42');
    is($Max, -23, 'ParseIntegerRange max -23');

    ($Min, $Max) = ParseIntegerRange('123456');
    ok($Min == $Max && $Min == 123456, 'ParseIntegerRange min=max');

    # invalid range
    eval { ok(!defined(ParseIntegerRange('23--42')), 'invalid range 1'); };
    ok(GetError()->code eq 'INVALID_RANGE', 'invalid range 1') if (ExistsError());

    # invalid range
    eval { ok(!defined(ParseIntegerRange('42-23')), 'invalid range 2'); };
    ok(GetError()->code eq 'INVALID_RANGE', 'invalid range 2') if (ExistsError());

    # invalid format
    eval { ok(!defined(ParseIntegerRange('42foo')), 'invalid format 1'); };
    ok(GetError()->code eq 'INVALID_FORMAT', 'invalid format 1') if (ExistsError());

    # invalid format
    eval { ok(!defined(ParseIntegerRange('42+23')), 'invalid format 2'); };
    ok(GetError()->code eq 'INVALID_FORMAT', 'invalid format 2') if (ExistsError());

    ok(ElementOfIntegerRange(23, '0-42'), 'ElementOfIntegerRange 1');
    ok(!ElementOfIntegerRange(666, '0-42'), 'ElementOfIntegerRange 2');
}

sub _TestIntegerList {
    my @List = ParseIntegerList('23-30,1-10,99');
    my @Expected = (23, 24, 25, 26, 27, 28, 29, 30, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 99);
    is(scalar(@List), scalar(@Expected), 'ParseIntegerList 1');
    my $Errors = 0;
    for(0 .. scalar(@List) - 1) {
        ++$Errors unless($List[$_] == $Expected[$_]);
    }
    is($Errors, 0, 'ParseIntegerList 2');

    @List = ParseIntegerList('23');
    ok(scalar(@List) == 1 && $List[0] == 23, 'ParseIntegerList 3');

    # invalid format
    eval { ok(!defined(ParseIntegerList('foo,bar,42')), 'invalid list format'); };
    ok(GetError()->code eq 'INVALID_FORMAT', 'invalid list format') if (ExistsError());

    # invalid range
    eval { ok(!defined(ParseIntegerList('42-23')), 'invalid range in list'); };
    ok(GetError()->code eq 'INVALID_RANGE', 'invalid range in list') if (ExistsError());

    ok(ElementOfIntegerList(23, '1,2,3,23,42,666'), 'ElementOfIntegerList 1');
    ok(!ElementOfIntegerList(23, '1,2,3,-23,42,666'), 'ElementOfIntegerList 2');
}

RunScript(Sub => sub {
    _TestIntegerRange();
    _TestIntegerList();
});
